//
//  ColorPlane.swift

import Foundation

final class ColorPlane {
    var width: Int
    var height: Int
    var maxHits: Int
    var data: [Int]

    init(width: Int, height: Int) {
        self.width = width
        self.height = height
        self.maxHits = 0
        self.data = [Int](count: width * height, repeatedValue: 0)
    }

    func increment(x: Int, y: Int) {
        data[width * y + x]++
        let m = data[width * y + x]
        if m > maxHits {maxHits = m}
    }

    func normalized() -> ColorPlane {
        let multiplier = 255.0 / log(Double(maxHits))
        var cp = ColorPlane(width: width, height: height)
        cp.data = data.map() {
            if $0 < 1 {return 0}
            return Int(log(Double($0)) * multiplier)
        }
        return cp
    }
}