//
//  Orbitals.swift
//

import Foundation
import Accelerate

class Orbitals : Mandelbrot {
    var cp:ColorPlane

    init(width:Int, zoom:Double, centerX:Double, centerY:Double, iterations:Int) {
        cp = ColorPlane(width: width, height: width)
        super.init(width: width, height: width, zoom: zoom, centerX: centerX, centerY: centerY, iterations: iterations)
        scanLines = [[Double]]()
    }

    // Use vector processing to compute Mandelbrot Set
    func computeOrbitalsLine(complex: DoubleSplitComplexVector, iterations: Int) -> DoubleSplitComplexVector {
        let len = vDSP_Length(complex.real.count)
        var intermediateResult = [DoubleSplitComplexVector]()
        var result = DoubleSplitComplexVector()
        var c_vector = complex
        var c = DSPDoubleSplitComplex(realp: &c_vector.real, imagp: &c_vector.imag)
        var z_vector = DoubleSplitComplexVector(length: complex.real.count)
        var z = DSPDoubleSplitComplex(realp: &z_vector.real, imagp: &z_vector.imag)
        var r_vector = DoubleSplitComplexVector(length: complex.real.count)
        var r = DSPDoubleSplitComplex(realp: &r_vector.real, imagp: &r_vector.imag)
        var d = [Double](count: complex.real.count, repeatedValue: 0.0)

        for i in 1 ... iterations {
            vDSP_zvmulD (&z, 1, &z, 1, &r, 1,len, 1)
            vDSP_zvaddD(&r, 1, &c, 1, &z, 1, len)
            var iterationVector = DoubleSplitComplexVector()
            // can't find a copy method for arrays so use map
            iterationVector.real = z_vector.real.map(){return $0}
            iterationVector.imag = z_vector.imag.map(){return $0}
            intermediateResult.append(iterationVector)
        }

        vDSP_vdistD(z.realp, 1, z.imagp, 1, &d, 1, len)

        for i in 0 ..< pixelWidth {
            let r = d[i]
            if !(r.isNaN) && r <= 2.0 {
                let rvec = intermediateResult.map(){return $0.real[i]}
                let ivec = intermediateResult.map(){return $0.imag[i]}
                result.real += rvec
                result.imag += ivec
            }
        }

        return result
    }

    func map(d: Double) -> Int {
        return max(min(Int(0.25 * (d + 2.0) * Double(pixelWidth)), pixelWidth - 1), 0)
    }

    override func computeSet() {
        for line in 0 ..< pixelHeight {
            threads.runOnComputationThread() {
                var splitComplexVector = DoubleSplitComplexVector()
                var imagVec = [Double](count: self.pixelWidth, repeatedValue: self.iArray[line])
                splitComplexVector.real = self.rArray
                splitComplexVector.imag = imagVec
                let scv = self.computeOrbitalsLine(splitComplexVector, iterations: self.iterations)
                if scv.real.count > 0 {
                    threads.runOnSerialThread() {
                        for i in 0 ..< scv.real.count {
                            self.cp.increment(self.map(scv.real[i]), y: self.map(scv.imag[i]))
                        }
                        println("Line: \(line)")
                    }
                }
            }
        }
        threads.waitAll()
        println("Done!")
    }

}