//
//  main.swift

import Foundation

func main() {
    computeOrbitals()
    exit(0)
}

func computeOrbitals() {
    let width = 1080
    let centerX = 0.0
    let centerY = 0.0
    let iterations = 2000
    let zoom = 1.0

    var fractalData = Orbitals(width: width, zoom: zoom, centerX: centerX, centerY: centerY, iterations: iterations)
    var image = Bitmap(width: width, height: width)

    fractalData.computeSet()
    let cp = fractalData.cp.normalized()

    var index = 0
    for c in cp.data {
        image.data[index++] = 0
        image.data[index++] = UInt8(min(c, 255))
        image.data[index++] = 0
        index++
    }

    println("Creating Image")
    image.saveImage("/Users/david/Desktop/testImage.png", type: "PNG")

    println("Done!")
}

func computeTraditional() {
    let width = 1280
    let height = 720
    let centerX = 0.75
    let centerY = 0.0
    let iterations = 4000
    let zoom = 1.5

    func mapSetValueToByte (d: Double) -> UInt8 {
        if d.isNaN || d.isInfinite { return 0 }
        let v = 128.0 * d
        if v < 0.0 { return 0 }
        if v > 255.0 { return 255 }
        return UInt8(v)
    }

    func mapOutsideValueToByte (d: Double) -> UInt8 {
        return UInt8(min((log(d) * 16.0 / log(Double(iterations)) * 16 + 112), 255))
    }
    
    var fractalData = Mandelbrot(width: width, height: height, zoom: zoom, centerX: centerX, centerY: centerY, iterations: iterations)
    var image = Bitmap(width: width, height: height)

    fractalData.computeSet()

    var imageData = [UInt8](count: width * height * 4, repeatedValue: 255)
    var index = 0
    for scanLine in fractalData.scanLines {
        for value in scanLine {
            if value < 2.0 {
                imageData[index++] = 0x2f
                imageData[index++] = 0x2f
                imageData[index++] = mapSetValueToByte(value)
                index++
            } else {
                imageData[index++] = mapOutsideValueToByte(value)
                imageData[index++] = 0xdf
                imageData[index++] = 0xdf
                index++
            }
        }
    }

    println("Creating Image")
    image.setColorData(imageData)
    image.saveImage("/Users/david/Desktop/testImage.jpg", type: "JPEG")

    println("Done!")
}

threads.runOnMainThread(main)
threads.run()
